#!/bin/sh
#
# Script to build production distribution of mpgedit
#
# Build directions:
# 
# 1) cd to desired build area
# 2) run make_dist.sh to fetch source, and compile for Linux
# 3) run compile for RedHat Linux 7x and Win32 platforms
# 4) su root
# 5) run make_dist.sh again to build distribution files

PATH=$PATH:/opt/bin
export PATH

VERSIONFILE="/home/number6/wrk/mpgedit/version.h"
if [ ! -f "$VERSIONFILE" ]; then
  tmp=`echo $0 | sed 's|\(.*\)/.*|\1|'`
  VERSIONFILE="$tmp/version.h"
fi
if [ ! -f "$VERSIONFILE" ]; then
  echo "ERROR: unable to find version.h file; quitting"
  exit 1
fi

RELEASE=`grep ' MPGEDIT_VERSION ' $VERSIONFILE  |      \
         sed -e 's/#define  *MPGEDIT_VERSION  *//' \
             -e 's/[ "-]//g' \
             -e 's/\./-/'`
ALPHA=`echo $RELEASE | sed -e 's/0-7//'`
TAG="mpgedit_0-7_beta_bugfixes-BP"

if [ -z "$RELEASE" ]; then
  RELEASE="0-7beta"
fi

echo TAG=$TAG
echo RELEASE=$RELEASE
echo
sleep 2

topdir="mpgedit_$RELEASE"
if [ ! -d "$topdir" ]; then
  echo
  echo "Creating new work area '$topdir/src/mpgedit'"
  echo "    ^C now to stop this operation"
  for i in 10 9 8 7 6 5 4 3 2 1; do
    echo -n "$i "
    sleep 1
  done

  mkdir "$topdir"
  cd $topdir
  if [ ! -d src ]; then
    mkdir src
    cd src
    cvs co -r $TAG mpgedit
  fi
  cd mpgedit
else
  if [ -d aix -o -d hpux10 ]; then
    echo "ERROR: You are running this script from the wrong directory"
    echo "       cwd = `pwd`"
    exit 1
  fi
  cd $topdir/src/mpgedit
fi

dir=../..
if [ ! -d $dir/linux ]; then
  ./man2html.sh
  (cd $dir; mkdir linux linux_rh7 solaris win32 hpux10 aix)
  (cd $dir/linux;     s_fetchtree -l -s ../src)
  (cd $dir/linux_rh7; s_fetchtree -l -s ../src)
  (cd $dir/solaris;   s_fetchtree -l -s ../src)
  (cd $dir/win32;     s_fetchtree -l -s ../src)
  (cd $dir/hpux10;    s_fetchtree -l -s ../src)
  (cd $dir/aix;       s_fetchtree -l -s ../src)
  (cd $dir/win32; unzip /opt/src/pdc24_vc_w32.zip)
  (cd $dir/win32/mpgedit; ln -s ../pdcurses/curses.dll .)
  (cd $dir/win32; unzip /home/number6/html/mpgedit/download/gtk-2.2.1-dll.zip)
  (cd $dir/linux/mpgedit; ./configure)
  echo "You must perform platform specific builds for win32 and solaris"
  echo "Afterwards, execute this script again"
  exit 0
fi

if [ ! -f $dir/linux/mpgedit/mpgedit -o \
     ! -f $dir/win32/mpgedit/mpgedit.exe ]; then
  
  echo "You must perform platform specific builds for win32 and solaris"
  echo "Afterwards, execute this script again"
  exit 0
fi

uid=`id | awk '{print $1}' | sed -e 's/uid=//' -e 's/(.*//'`
if [ $uid -ne 0 ]; then
  echo "you must be root to continue executing from this point"
  exit 1
fi


#
# Build Win32 SDK
#
#(cd $dir/win32/mpgedit;     ./make_sdk.sh       $RELEASE)

#
# Build the distribution packages areas
#
(cd $dir/linux/mpgedit;     ./dist_linux.sh     $RELEASE)
(cd $dir/linux_rh7/mpgedit; ./dist_linux_rh7.sh $RELEASE)
(cd $dir/win32/mpgedit;     ./dist_win32.sh     $RELEASE)
./dist_src.sh "$RELEASE" "$TAG"

#
# Tar/zip them up
#
(
if [ ! -s $dir/mpgedit_${RELEASE}_linux.tar.gz ]; then
  cd $dir; tar zcf mpgedit_${RELEASE}_linux.tar.gz mpgedit_${RELEASE}_linux
fi
)
(
if [ ! -s $dir/mpgedit_${RELEASE}_linux_rh7.tar.gz ]; then
  cd $dir; tar zcf mpgedit_${RELEASE}_linux_rh7.tar.gz mpgedit_${RELEASE}_linux_rh7
fi
)
(
if [ ! -s $dir/mpgedit_${RELEASE}_src.tar.gz ]; then
  cd $dir; tar zcf mpgedit_${RELEASE}_src.tar.gz mpgedit_${RELEASE}_src
fi
)
(
if [ ! -s $dir/mpgedit_${RELEASE}_win32.zip ]; then
  cd $dir; zip -r mpgedit_${RELEASE}_win32.zip mpgedit_${RELEASE}_win32
fi
)

#
# Copy these distribution files to the local copy of mpgedit web site
# 
(

  cat NEWS | sed -e '1i\
<HTML><pre>'     -e '$a\
</pre>' > /tmp/NEWS_0_7.html

  cd $dir
  cat << NNNN > cpit.sh
su number6 -c 'cp mpgedit_${RELEASE}_linux.tar.gz /home/number6/html/mpgedit/download'
su number6 -c 'cp mpgedit_${RELEASE}_linux_rh7.tar.gz /home/number6/html/mpgedit/download'
su number6 -c 'cp mpgedit_${RELEASE}_src.tar.gz /home/number6/html/mpgedit/download'
su number6 -c 'cp mpgedit_${RELEASE}_win32.zip /home/number6/html/mpgedit/download'
su number6 -c 'cp win32/mpgedit/gui/Output/setup.exe /home/number6/html/mpgedit/download/setup_xmpgedit_${RELEASE}.exe'
su number6 -c 'cp win32/mpgedit/Output/setup.exe /home/number6/html/mpgedit/download/setup_mpgedit_${RELEASE}.exe'
#su number6 -c 'cp win32/mpgedit/mpgedit_sdk.zip /home/number6/html/mpgedit/download'
su number6 -c 'cp /tmp/NEWS_0_7.html /home/number6/html/mpgedit'
NNNN
  sh -v cpit.sh
  rm -f /tmp/NEWS_0_7.html

  # Update downloads.html page to contain new alpha version.
  # This has become complicated enough that a real sed script
  # is needed to do this work.
  #
  cat<<NNNN>/tmp/sedscript$$
s|>Last modified: .*<|>Last modified: `date +%x`<|
s|[0-9]-[0-9]*alpha[0-9][0-9]*|$ALPHA|
/[0-9]-[0-9]*alpha-[0-9][0-9]*/{
  s|[0-9]-[0-9]*alpha-[0-9][0-9]*|$ALPHA|
  s|\(alpha\)\([0-9][0-9]*\)|\1-\2|
}
/setup_xmpgedit_.*.exe/{
  s|setup_xmpgedit_.*\.exe|setup_xmpgedit_${RELEASE}.exe|
}
NNNN

  # Run the generated sed script above
  #
  cat /home/number6/html/mpgedit/downloads.html | \
    sed -f /tmp/sedscript$$ > /tmp/downloads.html.edited

  if [ -s /tmp/downloads.html.edited ]; then
    cp /tmp/downloads.html.edited /home/number6/html/mpgedit/downloads.html
    cp /tmp/downloads.html.edited /home/number6/html/mpgedit
  fi

  # Update home.html page to contain new alpha version
  #
  cat /home/number6/html/mpgedit/home.html | \
    sed -e "s|>Last modified: .*<|>Last modified: `date +%x`<|" \
        -e "s|alpha[0-9][0-9]*|$ALPHA|" > \
    /tmp/home.html.edited
  if [ -s /tmp/home.html.edited ]; then
    cp /tmp/home.html.edited /home/number6/html/mpgedit/home.html
    cp /tmp/home.html.edited /home/number6/html/mpgedit
  fi
)
