#ifndef _PLUGIN_H
#define _PLUGIN_H
#include <stdio.h>


typedef void (*void_fp_t)(void);
typedef void *(*mpgdecoder_alloc_t)(void);
typedef void (*mpgdecoder_free_t)(void *);
typedef void (*mpgdecoder_init_t)(void *);
typedef FILE *(*mpgdecoder_open_t)(void *, int, int);
typedef void (*mpgdecoder_close_t)(void *);
typedef void (*mpgdecoder_play_frame_t)
         (void *, FILE **, unsigned char *, int, int, int);
typedef char *(*mpgdecoder_api_ver_06_t)(void);
typedef int (*mpgdecoder_decode_frame_t) 
        (void *,
         unsigned char *, int ,
         unsigned char **, int *);

/*
 * This union is needed to allow dlsym() assignment  without
 * casting required, and to preserve type checking.  dlsym() 
 * always assigns to voidcast entry.
 */
typedef union _plugin_fps {
    void_fp_t               voidcast;
    mpgdecoder_alloc_t      alloc;
    mpgdecoder_free_t       free;
    mpgdecoder_init_t       init;
    mpgdecoder_open_t       open;
    mpgdecoder_close_t      close;
    mpgdecoder_play_frame_t play;
    mpgdecoder_api_ver_06_t version;
    mpgdecoder_decode_frame_t decode;
} plugin_fps;


typedef struct _plugin_fcn_desc {
    char       *name;
    plugin_fps func;
} plugin_fcn_dsc;


/*
 * These definitions must always be in the same order as the 
 * function names when initialized in mpgfio_defaults_decoder_plugin()
 */
#define fp_mpgdecoder_alloc       (fp_mpgdecoder[0].func.alloc)
#define fp_mpgdecoder_free        (fp_mpgdecoder[1].func.free)
#define fp_mpgdecoder_init        (fp_mpgdecoder[2].func.init)
#define fp_mpgdecoder_open        (fp_mpgdecoder[3].func.open)
#define fp_mpgdecoder_close       (fp_mpgdecoder[4].func.close)
#define fp_mpgdecoder_play_frame  (fp_mpgdecoder[5].func.play)
#define fp_mpgdecoder_api_ver_06  (fp_mpgdecoder[6].func.version)

/* Extension functions */
#define fp_mpgdecoder_decode_frame (fp_mpgdecoder[7].func.decode)

plugin_fcn_dsc *mpgfio_alloc_decoder_plugin(void);
void mpgfio_defaults_decoder_plugin(plugin_fcn_dsc *f);
char *mpegfio_load_decoder_plugin(char *file, plugin_fcn_dsc *f);

#endif

