/*
 * XING header manipulation routines
 *
 * Copyright (C) 2001, 2002, Adam Bernstein
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307  USA.
 */


#ifndef lint
static char SccsId[] = "$Id: xing_header.c,v 1.8 2003/09/11 06:14:12 number6 Exp $";
#endif

#include <stdio.h>
#include <string.h>
#include "xing_header.h"
#ifdef __do_graffiti
#include "version.h"
#include "mpgedit_buildnum.h"
#endif

#if 0
#define UNIT_TEST 1
#endif

static int sr_table[4] = { 44100, 48000, 32000, 99999 };

/*  big endian insert */

void InsertI4(int x, unsigned char *buf)
{
    buf[3] = x & 0xff;
    x >>= 8;

    buf[2] = x & 0xff;
    x >>= 8;

    buf[1] = x & 0xff;
    x >>= 8;

    buf[0] = x & 0xff;
}

/*  big endian extract */

int ExtractI4(unsigned char *buf)
{
    int x;

    x = buf[0];
    x <<= 8;
    x |= buf[1];
    x <<= 8;
    x |= buf[2];
    x <<= 8;
    x |= buf[3];

    return x;
}
/*
 * Poke new values for frames and file size into an already populated
 * and valid Xing header. 'version' is the raw mpeg version value
 * from the decoded header. 'mode' is the raw mpeg channel mode value
 * from the decoded header.
 */
static void edit_xing_header(int frames, int bytes, 
                      unsigned char *buf, int len,
                      int version, int mode)
{
    int offset = 44;
#ifdef __do_graffiti
    int i;
    char graffiti[128];
#endif

    switch (version) {
      case 3:                /* mpeg 1 */
        switch (mode) {
          case 3:            /* mono */
            offset = 29;
            break;
          default:
            offset = 44;     /* stereo/joint/dual channel */
            break;
        }
        break;
      default:               /* mpeg 2/2.5 */
        switch (mode) {
          case 3:            /* mono */
            offset = 21;
            break;
          default:           /* stereo/joint/dual channel */
            offset = 29;
            break;
        }
        break;
    }
    InsertI4(frames, &buf[offset]);
    InsertI4(bytes,  &buf[offset+4]);

#ifdef __do_graffiti
/*
 * adam/TBD: Not ready to put this into production. This change
 * invalidates the regression test scripts, because the split/spliced
 * file is no longer identical to the original.
 */
    /*
     * Add mpgedit "graffiti"
     */
    sprintf(graffiti, "%s %s",
            MPGEDIT_PRODUCT,
            MPGEDIT_VERSION);

    /*
     * Count how much zeroed space is present from the
     * end of frame to the first non-zero byte. Does
     */
    for (i=len-1; i>0 && !buf[i]; i--)
        ;

    i+= 2;
    if (strlen(graffiti) < len-1) {
        /*
         * Graffiti fits in this empty space.
         * Yes, the NULL is NOT copied
         */
        memcpy(buf+i, graffiti, strlen(graffiti));
    }
#endif
}


int xingheader_file_make(char *file)
{
    static unsigned char xing_header[] = {
      0xff, 0xfb, 0x50, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x00, 0x58, 0x69, 0x6e, 0x67, 0x00, 0x00, 0x00, 0x0f,
      0x00, 0x00, 0x04, 0xf0, 0x00, 0x09, 0x19, 0x6c, 0x00, 0x02, 0x04,
      0x06, 0x08, 0x0a, 0x0d, 0x0f, 0x12, 0x15, 0x17, 0x1a, 0x1d, 0x20,
      0x23, 0x26, 0x29, 0x2c, 0x2f, 0x32, 0x34, 0x37, 0x3a, 0x3c, 0x3f,
      0x41, 0x43, 0x45, 0x48, 0x4b, 0x4e, 0x50, 0x53, 0x56, 0x59, 0x5c,
      0x5f, 0x62, 0x64, 0x66, 0x69, 0x6b, 0x6e, 0x71, 0x73, 0x75, 0x77,
      0x79, 0x7b, 0x7e, 0x80, 0x82, 0x85, 0x87, 0x8a, 0x8c, 0x8f, 0x91,
      0x93, 0x96, 0x98, 0x9a, 0x9c, 0x9f, 0xa1, 0xa3, 0xa6, 0xa7, 0xa9,
      0xac, 0xae, 0xb1, 0xb3, 0xb6, 0xb9, 0xbc, 0xbf, 0xc2, 0xc5, 0xc7,
      0xca, 0xcc, 0xcf, 0xd2, 0xd4, 0xd7, 0xd9, 0xdc, 0xde, 0xe1, 0xe4,
      0xe6, 0xe9, 0xec, 0xef, 0xf3, 0xf6, 0xf9, 0xfc, 0xfe, 0x00, 0x00,
      0x00, 0x42, 0x4c, 0x41, 0x4d, 0x45, 0x33, 0x2e, 0x35, 0x30, 0x00,
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };

    FILE *fp;

    /*
     * Test for file existence.  Done when exists.
     */
    fp = fopen(file, "rb");
    if (fp) {
        fclose(fp);
        return 0;
    }

    fp = fopen(file, "wb");
    if (!fp) {
        return -1;
    }

    fwrite(xing_header, sizeof(xing_header), 1, fp);
    fclose(fp);
    return 0;
}


/* Return TRUE if valid Xing header is found, FALSE otherwise */

static int get_xing_header(unsigned char *buf, XHEADDATA *X)
{
    int h_id;
    int h_id2;
    int h_layer;
    int h_protect;
    int h_sr_index;
    int h_mode;
    int head_flags;
    int i;

    /* 11111111 111BBCCD EEEEFFGH IIJJKLMM  */

    /* BB */
    h_id       = (buf[1] >> 3) & 1;
    h_id2      = (buf[1] >> 3) & 2;

    /* CC */
    h_layer    = (buf[1] >> 1) & 3;

    /* D */
    h_protect  =  buf[1]       & 1;

    h_sr_index = (buf[2] >> 2) & 3;
    h_mode     = (buf[3] >> 6) & 3;
    X->h_mode  = h_mode;

    if (h_id) {        /*  mpeg1 */
        if (h_mode != 3) {
            buf += (32+4);
        }
        else {
            buf += (17+4);
        }
    }
    else {      /* mpeg2 */
        if (h_mode != 3) {
            buf += (17+4);
        }
        else {
            buf += (9+4);
        }
    }

    if (h_layer == 3) {
        X->h_layer = 1;
    }
    else if (h_layer == 2) {
        X->h_layer = 2;
    }
    else if (h_layer == 1) {
        X->h_layer = 3;
    }
    else {
        X->h_layer = 0;
    }

    X->h_protect = !h_protect;

    /* fail, header not found */

    if (buf[0] != 'X') return 0;
    if (buf[1] != 'i') return 0;
    if (buf[2] != 'n') return 0;
    if (buf[3] != 'g') return 0;
    buf+=4;

    X->h_id = h_id | h_id2;
    X->samprate = sr_table[h_sr_index];
    if (h_id == 0) {
        X->samprate >>= 1;
        if (h_id2 == 0) X->samprate >>= 1;
    }

    /* get flags */

    head_flags = X->flags = ExtractI4(buf);
    buf+=4;

    if (head_flags & FRAMES_FLAG) {
        X->frames  = ExtractI4(buf);
        buf+=4;
    }
    if (head_flags & BYTES_FLAG) {
        X->bytes = ExtractI4(buf);
        buf+=4;
    }

    if (head_flags & TOC_FLAG) {
        if (X->toc != NULL) {
            for(i=0;i<100;i++) {
                X->toc[i] = buf[i];
            }
        }
        buf+=100;
    }

    X->vbr_scale = -1;
    if (head_flags & VBR_SCALE_FLAG)  {
       X->vbr_scale = ExtractI4(buf); buf+=4;
    }
    return 1;
}


static void xing_header2str(XHEADDATA *X, char *cp)
{
    cp += sprintf(cp, "h_id      = %d\n", X->h_id);
    cp += sprintf(cp, "h_layer   = %d\n", X->h_layer);
    cp += sprintf(cp, "h_protect = %d\n", X->h_protect);
    cp += sprintf(cp, "samprate  = %d\n", X->samprate);
    cp += sprintf(cp, "flags     = %d\n", X->flags);
    cp += sprintf(cp, "frames    = %d\n", X->frames);
    cp += sprintf(cp, "bytes     = %d\n", X->bytes);
    cp += sprintf(cp, "vbr_scale = %d\n", X->vbr_scale);
}


int xingheader_init(unsigned char *buf, int len, XHEADDATA *X)
{
    XHEADDATA xingh;
    int rval;

    if (!X) {
        memset(&xingh, 0, sizeof(xingh));
        memcpy(xingh.xingbuf, buf, len);
        xingh.xingbuflen = len;
        rval = get_xing_header(buf, &xingh);
    }
    else {
        if (buf != X->xingbuf) {
            memmove(X->xingbuf, buf, len);
        }
        X->xingbuflen = len;
        rval = get_xing_header(buf, X);
    }
    return rval;
}


void xingheader_edit(XHEADDATA *xingh, int frames, int bytes, int id, int mode)
{
    edit_xing_header(frames, bytes, xingh->xingbuf, 
                     xingh->xingbuflen, id, mode);
    get_xing_header(xingh->xingbuf, xingh);
}


void xingheader2str(XHEADDATA *xingh, char *cp)
{
    xing_header2str(xingh, cp);
}



#if defined(UNIT_TEST)
#include <stdlib.h>
int main(int argc, char *argv[])
{
#define MAX_XING_HEADER_SIZE 2048
    unsigned char buf[MAX_XING_HEADER_SIZE];
    FILE *fp;
    int items;
    XHEADDATA X;
    int i;
    int frame_add = 0;
    int bytes_add = 0;
    int frames = 0;
    int bytes = 0;
    char *filename;
    char *cp;

    memset(buf, 0, sizeof(buf));
    InsertI4(0x010f03f0, buf); 

    memset(&X, 0, sizeof(X));

    if (argc == 1 || argc > 4) {
        printf("usage: %s mp3_file [[+]frames] [[+]bytes]\n", argv[0]);
        return 1;
    }
    filename = argv[1];

    if (argc > 2) {
        cp = argv[2];
        if (*cp == '+') {
            frame_add = 1;
            cp++;
        }
        frames = atoi(cp);
    }
    if (argc == 4) {
        cp = argv[3];
        if (*cp == '+') {
            bytes_add = 1;
            cp++;
        }
        bytes = atoi(cp);
    }

    fp = fopen(filename, "rb+");
    if (!fp) {
        perror("fopen");
        return 1;
    }
    items = fread(buf, 1, MAX_XING_HEADER_SIZE, fp);

    for (i = 0; i < 64; i++) {
        if ((i%16) == 0) {
            printf("\n");
        }
        printf("%02x ", buf[i]);
    }
    printf("\n");
    xingheader_init(buf, items, &X);

    printf("h_id      = %d\n", X.h_id);
    printf("h_layer   = %d\n", X.h_layer);
    printf("h_protect = %d\n", X.h_protect);
    printf("samprate  = %d\n", X.samprate);
    printf("flags     = %d\n", X.flags);
    printf("frames    = %d\n", X.frames);
    printf("bytes     = %d\n", X.bytes);
    printf("vbr_scale = %d\n", X.vbr_scale);
    printf("==================================\n");

    if (frame_add) {
        frames += X.frames;
    }
    if (bytes_add) {
        bytes += X.bytes;
    }
    if (frames) {
        if (bytes == 0) {
            bytes = X.bytes;
        }

        xingheader_edit(&X, frames, bytes, X.h_id, X.h_mode);
        xingheader_init(buf, items, &X);
        printf("h_id      = %d\n", X.h_id);
        printf("h_layer   = %d\n", X.h_layer);
        printf("h_protect = %d\n", X.h_protect);
        printf("samprate  = %d\n", X.samprate);
        printf("flags     = %d\n", X.flags);
        printf("frames    = %d\n", X.frames);
        printf("h_id      = %d\n", X.h_id);
        printf("h_layer   = %d\n", X.h_layer);
        printf("h_protect = %d\n", X.h_protect);
        printf("samprate  = %d\n", X.samprate);
        printf("flags     = %d\n", X.flags);
        printf("frames    = %d\n", X.frames);
        printf("bytes     = %d\n", X.bytes);
        printf("vbr_scale = %d\n", X.vbr_scale);
        printf("==================================\n");
        fseek(fp, 0, SEEK_SET);
        fwrite(buf, MAX_XING_HEADER_SIZE, 1, fp);
        fclose(fp);
    }
    return 0;
}
#endif
